@@ -23,7 +23,7 @@
 
 #ifndef lint
 static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-ospf.c,v 1.56.2.4 2006/12/13 08:24:27 hannes Exp $ (LBL)";
+    "@(#) $Header: /tcpdump/master/tcpdump/print-ospf.c,v 1.66 2007-10-08 07:53:21 hannes Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -44,11 +44,11 @@ static const char rcsid[] _U_ =
 #include "ip.h"
 
 static struct tok ospf_option_values[] = {
-	{ OSPF_OPTION_T,	"TOS" },
+        { OSPF_OPTION_T,	"MultiTopology" }, /* draft-ietf-ospf-mt-09 */
 	{ OSPF_OPTION_E,	"External" },
 	{ OSPF_OPTION_MC,	"Multicast" },
 	{ OSPF_OPTION_NP,	"NSSA" },
-	{ OSPF_OPTION_EA,	"Advertise External" },
+        { OSPF_OPTION_L,        "LLS" },
 	{ OSPF_OPTION_DC,	"Demand Circuit" },
 	{ OSPF_OPTION_O,	"Opaque" },
 	{ OSPF_OPTION_DN,	"Up/Down" },
@@ -98,6 +98,7 @@ static struct tok ospf_dd_flag_values[] = {
 	{ OSPF_DB_INIT,	        "Init" },
 	{ OSPF_DB_MORE,	        "More" },
 	{ OSPF_DB_MASTER,	"Master" },
+    { OSPF_DB_RESYNC,	"OOBResync" },
 	{ 0,			NULL }
 };
 
@@ -172,7 +173,19 @@ static struct tok lsa_opaque_ri_tlv_cap_values[] = {
 	{ 0,		        NULL }
 };
 
-static char tstr[] = " [|ospf]";
+static struct tok ospf_lls_tlv_values[] = {
+	{ OSPF_LLS_EO,	"Extended Options" },
+	{ OSPF_LLS_MD5,	"MD5 Authentication" },
+	{ 0,	NULL }
+};
+
+static struct tok ospf_lls_eo_options[] = {
+	{ OSPF_LLS_EO_LR,	"LSDB resync" },
+	{ OSPF_LLS_EO_RS,	"Restart" },
+	{ 0,	NULL }
+};
+
+static char tstr[] = " [|ospf2]";
 
 #ifdef WIN32
 #define inline __inline
@@ -181,6 +194,274 @@ static char tstr[] = " [|ospf]";
 static int ospf_print_lshdr(const struct lsa_hdr *);
 static const u_char *ospf_print_lsa(const struct lsa *);
 static int ospf_decode_v2(const struct ospfhdr *, const u_char *);
+static int ospf_decode_lls(const struct ospfhdr *, register u_int);
+
+int
+ospf_print_grace_lsa (u_int8_t *tptr, u_int ls_length) {
+
+    u_int tlv_type, tlv_length;
+
+
+    while (ls_length > 0) {
+        TCHECK2(*tptr, 4);
+        if (ls_length < 4) {
+            printf("\n\t    Remaining LS length %u < 4", ls_length);
+            return -1;
+        }
+        tlv_type = EXTRACT_16BITS(tptr);
+        tlv_length = EXTRACT_16BITS(tptr+2);
+        tptr+=4;
+        ls_length-=4;
+                    
+        printf("\n\t    %s TLV (%u), length %u, value: ",
+               tok2str(lsa_opaque_grace_tlv_values,"unknown",tlv_type),
+               tlv_type,
+               tlv_length);
+
+        if (tlv_length > ls_length) {
+            printf("\n\t    Bogus length %u > %u", tlv_length,
+                   ls_length);
+            return -1;
+        }
+
+        /* Infinite loop protection. */
+        if (tlv_type == 0 || tlv_length ==0) {
+            return -1;
+        }
+
+        TCHECK2(*tptr, tlv_length);
+        switch(tlv_type) {
+
+        case LS_OPAQUE_GRACE_TLV_PERIOD:
+            if (tlv_length != 4) {
+                printf("\n\t    Bogus length %u != 4", tlv_length);
+                return -1;
+            }
+            printf("%us",EXTRACT_32BITS(tptr));
+            break;
+
+        case LS_OPAQUE_GRACE_TLV_REASON:
+            if (tlv_length != 1) {
+                printf("\n\t    Bogus length %u != 1", tlv_length);
+                return -1;
+            }
+            printf("%s (%u)",
+                   tok2str(lsa_opaque_grace_tlv_reason_values, "Unknown", *tptr),
+                   *tptr);
+            break;
+
+        case LS_OPAQUE_GRACE_TLV_INT_ADDRESS:
+            if (tlv_length != 4) {
+                printf("\n\t    Bogus length %u != 4", tlv_length);
+                return -1;
+            }
+            printf("%s", ipaddr_string(tptr));
+            break;
+
+        default:
+            if (vflag <= 1) {
+                if(!print_unknown_data(tptr,"\n\t      ",tlv_length))
+                    return -1;
+            }
+            break;
+
+        }
+        /* in OSPF everything has to be 32-bit aligned, including TLVs */
+        if (tlv_length%4 != 0)
+            tlv_length+=4-(tlv_length%4);
+        ls_length-=tlv_length;
+        tptr+=tlv_length;
+    }
+
+    return 0;
+trunc:
+    return -1;
+}
+
+int
+ospf_print_te_lsa (u_int8_t *tptr, u_int ls_length) {
+
+    u_int tlv_type, tlv_length, subtlv_type, subtlv_length;
+    u_int priority_level, te_class, count_srlg;
+    union { /* int to float conversion buffer for several subTLVs */
+        float f; 
+        u_int32_t i;
+    } bw;
+
+    while (ls_length != 0) {
+        TCHECK2(*tptr, 4);
+        if (ls_length < 4) {
+            printf("\n\t    Remaining LS length %u < 4", ls_length);
+            return -1;
+        }
+        tlv_type = EXTRACT_16BITS(tptr);
+        tlv_length = EXTRACT_16BITS(tptr+2);
+        tptr+=4;
+        ls_length-=4;
+                    
+        printf("\n\t    %s TLV (%u), length: %u",
+               tok2str(lsa_opaque_te_tlv_values,"unknown",tlv_type),
+               tlv_type,
+               tlv_length);
+
+        if (tlv_length > ls_length) {
+            printf("\n\t    Bogus length %u > %u", tlv_length,
+                   ls_length);
+            return -1;
+        }
+
+        /* Infinite loop protection. */
+        if (tlv_type == 0 || tlv_length ==0) {
+            return -1;
+        }
+
+        switch(tlv_type) {
+        case LS_OPAQUE_TE_TLV_LINK:
+            while (tlv_length >= sizeof(subtlv_type) + sizeof(subtlv_length)) {
+                if (tlv_length < 4) {
+                    printf("\n\t    Remaining TLV length %u < 4",
+                           tlv_length);
+                    return -1;
+                }
+                TCHECK2(*tptr, 4);
+                subtlv_type = EXTRACT_16BITS(tptr);
+                subtlv_length = EXTRACT_16BITS(tptr+2);
+                tptr+=4;
+                tlv_length-=4;
+                            
+                printf("\n\t      %s subTLV (%u), length: %u",
+                       tok2str(lsa_opaque_te_link_tlv_subtlv_values,"unknown",subtlv_type),
+                       subtlv_type,
+                       subtlv_length);
+                            
+                TCHECK2(*tptr, subtlv_length);
+                switch(subtlv_type) {
+                case LS_OPAQUE_TE_LINK_SUBTLV_ADMIN_GROUP:
+                    printf(", 0x%08x", EXTRACT_32BITS(tptr));
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_LINK_ID:
+                case LS_OPAQUE_TE_LINK_SUBTLV_LINK_LOCAL_REMOTE_ID:
+                    printf(", %s (0x%08x)",
+                           ipaddr_string(tptr),
+                           EXTRACT_32BITS(tptr));
+                    if (subtlv_length == 8) /* rfc4203 */
+                        printf(", %s (0x%08x)",
+                               ipaddr_string(tptr+4),
+                               EXTRACT_32BITS(tptr+4));
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_LOCAL_IP:
+                case LS_OPAQUE_TE_LINK_SUBTLV_REMOTE_IP:
+                    printf(", %s", ipaddr_string(tptr));
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_MAX_BW:
+                case LS_OPAQUE_TE_LINK_SUBTLV_MAX_RES_BW:
+                    bw.i = EXTRACT_32BITS(tptr);
+                    printf(", %.3f Mbps", bw.f*8/1000000 );
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_UNRES_BW:
+                    for (te_class = 0; te_class < 8; te_class++) {
+                        bw.i = EXTRACT_32BITS(tptr+te_class*4);
+                        printf("\n\t\tTE-Class %u: %.3f Mbps",
+                               te_class,
+                               bw.f*8/1000000 );
+                    }
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_BW_CONSTRAINTS:
+                    printf("\n\t\tBandwidth Constraints Model ID: %s (%u)",
+                           tok2str(diffserv_te_bc_values, "unknown", *tptr),
+                           *tptr);
+                    /* decode BCs until the subTLV ends */
+                    for (te_class = 0; te_class < (subtlv_length-4)/4; te_class++) {
+                        bw.i = EXTRACT_32BITS(tptr+4+te_class*4);
+                        printf("\n\t\t  Bandwidth constraint CT%u: %.3f Mbps",
+                               te_class,
+                               bw.f*8/1000000 );
+                    }
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_TE_METRIC:
+                    printf(", Metric %u", EXTRACT_32BITS(tptr));
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_LINK_PROTECTION_TYPE:
+                    printf(", %s, Priority %u",
+                           bittok2str(gmpls_link_prot_values, "none", *tptr),
+                           *(tptr+1));
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_INTF_SW_CAP_DESCR:
+                    printf("\n\t\tInterface Switching Capability: %s",
+                           tok2str(gmpls_switch_cap_values, "Unknown", *(tptr)));
+                    printf("\n\t\tLSP Encoding: %s\n\t\tMax LSP Bandwidth:",
+                           tok2str(gmpls_encoding_values, "Unknown", *(tptr+1)));
+                    for (priority_level = 0; priority_level < 8; priority_level++) {
+                        bw.i = EXTRACT_32BITS(tptr+4+(priority_level*4));
+                        printf("\n\t\t  priority level %d: %.3f Mbps",
+                               priority_level,
+                               bw.f*8/1000000 );
+                    }
+                    break;
+                case LS_OPAQUE_TE_LINK_SUBTLV_LINK_TYPE:
+                    printf(", %s (%u)",
+                           tok2str(lsa_opaque_te_tlv_link_type_sub_tlv_values,"unknown",*tptr),
+                           *tptr);
+                    break;
+
+                case LS_OPAQUE_TE_LINK_SUBTLV_SHARED_RISK_GROUP:
+                    count_srlg = subtlv_length / 4;
+                    if (count_srlg != 0)
+                        printf("\n\t\t  Shared risk group: ");
+                    while (count_srlg > 0) {
+                        bw.i = EXTRACT_32BITS(tptr);
+                        printf("%d",bw.i);
+                        tptr+=4;
+                        count_srlg--;
+                        if (count_srlg > 0)
+                            printf(", ");
+                    }
+                    break;
+
+                default:
+                    if (vflag <= 1) {
+                        if(!print_unknown_data(tptr,"\n\t\t",subtlv_length))
+                            return -1;
+                    }
+                    break;
+                }
+                /* in OSPF everything has to be 32-bit aligned, including subTLVs */
+                if (subtlv_length%4 != 0)
+                    subtlv_length+=4-(subtlv_length%4);
+                            
+                tlv_length-=subtlv_length;
+                tptr+=subtlv_length;
+                            
+            }
+            break;
+                        
+        case LS_OPAQUE_TE_TLV_ROUTER:
+            if (tlv_length < 4) {
+                printf("\n\t    TLV length %u < 4", tlv_length);
+                return -1;
+            }
+            TCHECK2(*tptr, 4);
+            printf(", %s", ipaddr_string(tptr));
+            break;
+                        
+        default:
+            if (vflag <= 1) {
+                if(!print_unknown_data(tptr,"\n\t      ",tlv_length))
+                    return -1;
+            }
+            break;
+        }
+        /* in OSPF everything has to be 32-bit aligned, including TLVs */
+        if (tlv_length%4 != 0)
+            tlv_length+=4-(tlv_length%4);
+        ls_length-=tlv_length;
+        tptr+=tlv_length;
+    }
+    return 0;
+trunc:
+    return -1;
+}
+
 
 static int
 ospf_print_lshdr(register const struct lsa_hdr *lshp)
@@ -190,13 +471,13 @@ ospf_print_lshdr(register const struct lsa_hdr *lshp)
         TCHECK(lshp->ls_length);
         ls_length = EXTRACT_16BITS(&lshp->ls_length);
         if (ls_length < sizeof(struct lsa_hdr)) {
-                printf("\n\t    Bogus length %u < %lu", ls_length,
+                printf("\n\t    Bogus length %u < header (%lu)", ls_length,
                     (unsigned long)sizeof(struct lsa_hdr));
                 return(-1);
         }
 
         TCHECK(lshp->ls_seq);	/* XXX - ls_length check checked this */
-	printf("\n\t  Advertising Router: %s, seq 0x%08x, age %us, length: %u",
+	printf("\n\t  Advertising Router %s, seq 0x%08x, age %us, length %u",
 	       ipaddr_string(&lshp->ls_router),
 	       EXTRACT_32BITS(&lshp->ls_seq),
 	       EXTRACT_16BITS(&lshp->ls_age),
@@ -208,7 +489,7 @@ ospf_print_lshdr(register const struct lsa_hdr *lshp)
         case LS_TYPE_OPAQUE_LL:
         case LS_TYPE_OPAQUE_AL:
         case LS_TYPE_OPAQUE_DW:
-            printf("\n\t    %s LSA (%d), Opaque-Type: %s LSA (%u), Opaque-ID: %u",
+            printf("\n\t    %s LSA (%d), Opaque-Type %s LSA (%u), Opaque-ID %u",
                    tok2str(lsa_values,"unknown",lshp->ls_type),
                    lshp->ls_type,
 
@@ -238,6 +519,41 @@ trunc:
 	return (-1);
 }
 
+/* draft-ietf-ospf-mt-09 */
+static struct tok ospf_topology_values[] = {
+    { 0, "default " },
+    { 1, "multicast " },
+    { 2, "management " },
+    { 0, NULL }
+};
+
+/*
+ * Print all the per-topology metrics.
+ */
+static void
+ospf_print_tos_metrics(const union un_tos *tos)
+{
+    int metric_count;
+    int toscount;
+
+    toscount = tos->link.link_tos_count+1;
+    metric_count = 0;
+
+    /*
+     * All but the first metric contain a valid topology id.
+     */
+    while (toscount) { 
+        printf("\n\t\ttopology %s(%u), metric %u",
+               tok2str(ospf_topology_values, "",
+                       metric_count ? tos->metrics.tos_type : 0),
+               metric_count ? tos->metrics.tos_type : 0,
+               EXTRACT_16BITS(&tos->metrics.tos_metric));
+        metric_count++;
+        tos++;
+        toscount--;
+    }
+}
+
 /*
  * Print a single link state advertisement.  If truncated or if LSA length
  * field is less than the length of the LSA header, return NULl, else
@@ -248,19 +564,13 @@ ospf_print_lsa(register const struct lsa *lsap)
 {
 	register const u_int8_t *ls_end;
 	register const struct rlalink *rlp;
-	register const struct tos_metric *tosp;
 	register const struct in_addr *ap;
 	register const struct aslametric *almp;
 	register const struct mcla *mcp;
 	register const u_int32_t *lp;
-	register int j, k, tlv_type, tlv_length, subtlv_type, subtlv_length, priority_level, te_class;
+	register int j, tlv_type, tlv_length, topology;
 	register int ls_length;
 	const u_int8_t *tptr;
-	int count_srlg;
-        union { /* int to float conversion buffer for several subTLVs */
-            float f; 
-            u_int32_t i;
-        } bw;
 
 	tptr = (u_int8_t *)lsap->lsa_un.un_unknown; /* squelch compiler warnings */
         ls_length = ospf_print_lshdr(&lsap->ls_hdr);
@@ -281,7 +591,7 @@ ospf_print_lsa(register const struct lsa *lsap)
 		rlp = lsap->lsa_un.un_rla.rla_link;
 		while (j--) {
 			TCHECK(*rlp);
-			switch (rlp->link_type) {
+			switch (rlp->un_tos.link.link_type) {
 
 			case RLA_TYPE_VIRTUAL:
 				printf("\n\t      Virtual Link: Neighbor Router-ID: %s, Interface Address: %s",
@@ -309,20 +619,14 @@ ospf_print_lsa(register const struct lsa *lsap)
 
 			default:
 				printf("\n\t      Unknown Router Link Type (%u)",
-				    rlp->link_type);
+				    rlp->un_tos.link.link_type);
 				return (ls_end);
 			}
-			printf(", tos 0, metric: %d", EXTRACT_16BITS(&rlp->link_tos0metric));
-			tosp = (struct tos_metric *)
-			    ((sizeof rlp->link_tos0metric) + (u_char *) rlp);
-			for (k = 0; k < (int) rlp->link_toscount; ++k, ++tosp) {
-				TCHECK(*tosp);
-				printf(", tos %d, metric: %d",
-				    tosp->tos_type,
-				    EXTRACT_16BITS(&tosp->tos_metric));
-			}
+
+                        ospf_print_tos_metrics(&rlp->un_tos);
+
 			rlp = (struct rlalink *)((u_char *)(rlp + 1) +
-			    ((rlp->link_toscount) * sizeof(*tosp)));
+			    ((rlp->un_tos.link.link_tos_count) * sizeof(union un_tos)));
 		}
 		break;
 
@@ -344,19 +648,16 @@ ospf_print_lsa(register const struct lsa *lsap)
 		    ipaddr_string(&lsap->lsa_un.un_sla.sla_mask));
 		TCHECK(lsap->lsa_un.un_sla.sla_tosmetric);
 		lp = lsap->lsa_un.un_sla.sla_tosmetric;
-                /* suppress tos if its not supported */
-                if(!((lsap->ls_hdr.ls_options)&OSPF_OPTION_T)) {
-                    printf(", metric: %u", EXTRACT_32BITS(lp)&SLA_MASK_METRIC);
-                    break;
-                }
 		while ((u_char *)lp < ls_end) {
 			register u_int32_t ul;
 
 			TCHECK(*lp);
 			ul = EXTRACT_32BITS(lp);
-			printf(", tos %d metric %d",
-			    (ul & SLA_MASK_TOS) >> SLA_SHIFT_TOS,
-			    ul & SLA_MASK_METRIC);
+                        topology = (ul & SLA_MASK_TOS) >> SLA_SHIFT_TOS;
+			printf("\n\t\ttopology %s(%u) metric %d",
+                               tok2str(ospf_topology_values, "", topology),
+                               topology,
+                               ul & SLA_MASK_METRIC);
 			++lp;
 		}
 		break;
@@ -364,19 +665,16 @@ ospf_print_lsa(register const struct lsa *lsap)
 	case LS_TYPE_SUM_ABR:
 		TCHECK(lsap->lsa_un.un_sla.sla_tosmetric);
 		lp = lsap->lsa_un.un_sla.sla_tosmetric;
-                /* suppress tos if its not supported */
-                if(!((lsap->ls_hdr.ls_options)&OSPF_OPTION_T)) {
-                    printf(", metric: %u", EXTRACT_32BITS(lp)&SLA_MASK_METRIC);
-                    break;
-                }
 		while ((u_char *)lp < ls_end) {
 			register u_int32_t ul;
 
 			TCHECK(*lp);
 			ul = EXTRACT_32BITS(lp);
-			printf(", tos %d metric %d",
-			    (ul & SLA_MASK_TOS) >> SLA_SHIFT_TOS,
-			    ul & SLA_MASK_METRIC);
+                        topology = (ul & SLA_MASK_TOS) >> SLA_SHIFT_TOS;
+			printf("\n\t\ttopology %s(%u) metric %d",
+                               tok2str(ospf_topology_values, "", topology),
+                               topology,
+                               ul & SLA_MASK_METRIC);
 			++lp;
 		}
 		break;
@@ -394,9 +692,11 @@ ospf_print_lsa(register const struct lsa *lsap)
 
 			TCHECK(almp->asla_tosmetric);
 			ul = EXTRACT_32BITS(&almp->asla_tosmetric);
-			printf(", type %d, tos %d metric:",
-			    (ul & ASLA_FLAG_EXTERNAL) ? 2 : 1,
-			    (ul & ASLA_MASK_TOS) >> ASLA_SHIFT_TOS);
+                        topology = ((ul & ASLA_MASK_TOS) >> ASLA_SHIFT_TOS);
+			printf("\n\t\ttopology %s(%u), type %d, metric",
+                               tok2str(ospf_topology_values, "", topology),
+                               topology,
+                               (ul & ASLA_FLAG_EXTERNAL) ? 2 : 1);
                         if ((ul & ASLA_MASK_METRIC)==0xffffff)
                             printf(" infinite");
                         else
@@ -471,7 +771,6 @@ ospf_print_lsa(register const struct lsa *lsap)
                             ls_length);
                         return(ls_end);
                     }
-                    ls_length-=tlv_length;
                     TCHECK2(*tptr, tlv_length);
                     switch(tlv_type) {
 
@@ -492,245 +791,32 @@ ospf_print_lsa(register const struct lsa *lsap)
 
                     }
                     tptr+=tlv_length;
+                    ls_length-=tlv_length;
                 }
-
                 break;
+
             case LS_OPAQUE_TYPE_GRACE:
-		tptr = (u_int8_t *)(&lsap->lsa_un.un_grace_tlv.type);
-
-		while (ls_length != 0) {
-                    TCHECK2(*tptr, 4);
-		    if (ls_length < 4) {
-                        printf("\n\t    Remaining LS length %u < 4", ls_length);
-                        return(ls_end);
-                    }
-                    tlv_type = EXTRACT_16BITS(tptr);
-                    tlv_length = EXTRACT_16BITS(tptr+2);
-                    tptr+=4;
-                    ls_length-=4;
-                    
-                    printf("\n\t    %s TLV (%u), length: %u, value: ",
-                           tok2str(lsa_opaque_grace_tlv_values,"unknown",tlv_type),
-                           tlv_type,
-                           tlv_length);
-
-                    if (tlv_length > ls_length) {
-                        printf("\n\t    Bogus length %u > %u", tlv_length,
-                            ls_length);
-                        return(ls_end);
-                    }
-                    ls_length-=tlv_length;
-                    TCHECK2(*tptr, tlv_length);
-                    switch(tlv_type) {
-
-                    case LS_OPAQUE_GRACE_TLV_PERIOD:
-                        if (tlv_length != 4) {
-                            printf("\n\t    Bogus length %u != 4", tlv_length);
-                            return(ls_end);
-                        }
-                        printf("%us",EXTRACT_32BITS(tptr));
-                        break;
-                    case LS_OPAQUE_GRACE_TLV_REASON:
-                        if (tlv_length != 1) {
-                            printf("\n\t    Bogus length %u != 1", tlv_length);
-                            return(ls_end);
-                        }
-                        printf("%s (%u)",
-                               tok2str(lsa_opaque_grace_tlv_reason_values, "Unknown", *tptr),
-                               *tptr);
-                        break;
-                    case LS_OPAQUE_GRACE_TLV_INT_ADDRESS:
-                        if (tlv_length != 4) {
-                            printf("\n\t    Bogus length %u != 4", tlv_length);
-                            return(ls_end);
-                        }
-                        printf("%s", ipaddr_string(tptr));
-                        break;
-                    default:
-                        if (vflag <= 1) {
-                            if(!print_unknown_data(tptr,"\n\t      ",tlv_length))
-                                return(ls_end);
-                        }
-                        break;
-
-                    }
-                    tptr+=tlv_length;
-                }
-
-                break;
-	    case LS_OPAQUE_TYPE_TE:
-		tptr = (u_int8_t *)(&lsap->lsa_un.un_te_lsa_tlv.type);
-
-		while (ls_length != 0) {
-                    TCHECK2(*tptr, 4);
-		    if (ls_length < 4) {
-                        printf("\n\t    Remaining LS length %u < 4", ls_length);
-                        return(ls_end);
-                    }
-                    tlv_type = EXTRACT_16BITS(tptr);
-                    tlv_length = EXTRACT_16BITS(tptr+2);
-                    tptr+=4;
-                    ls_length-=4;
-                    
-                    printf("\n\t    %s TLV (%u), length: %u",
-                           tok2str(lsa_opaque_te_tlv_values,"unknown",tlv_type),
-                           tlv_type,
-                           tlv_length);
-
-                    if (tlv_length > ls_length) {
-                        printf("\n\t    Bogus length %u > %u", tlv_length,
-                            ls_length);
-                        return(ls_end);
-                    }
-                    ls_length-=tlv_length;
-                    switch(tlv_type) {
-                    case LS_OPAQUE_TE_TLV_LINK:
-                        while (tlv_length != 0) {
-                            if (tlv_length < 4) {
-                                printf("\n\t    Remaining TLV length %u < 4",
-                                    tlv_length);
-                                return(ls_end);
-                            }
-                            TCHECK2(*tptr, 4);
-                            subtlv_type = EXTRACT_16BITS(tptr);
-                            subtlv_length = EXTRACT_16BITS(tptr+2);
-                            tptr+=4;
-                            tlv_length-=4;
-                            
-                            printf("\n\t      %s subTLV (%u), length: %u",
-                                   tok2str(lsa_opaque_te_link_tlv_subtlv_values,"unknown",subtlv_type),
-                                   subtlv_type,
-                                   subtlv_length);
-                            
-                            TCHECK2(*tptr, subtlv_length);
-                            switch(subtlv_type) {
-                            case LS_OPAQUE_TE_LINK_SUBTLV_ADMIN_GROUP:
-                                printf(", 0x%08x", EXTRACT_32BITS(tptr));
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_LINK_ID:
-                            case LS_OPAQUE_TE_LINK_SUBTLV_LINK_LOCAL_REMOTE_ID:
-                                printf(", %s (0x%08x)",
-                                       ipaddr_string(tptr),
-                                       EXTRACT_32BITS(tptr));
-                                if (subtlv_length == 8) /* draft-ietf-ccamp-ospf-gmpls-extensions */
-                                    printf(", %s (0x%08x)",
-                                           ipaddr_string(tptr+4),
-                                           EXTRACT_32BITS(tptr+4));
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_LOCAL_IP:
-                            case LS_OPAQUE_TE_LINK_SUBTLV_REMOTE_IP:
-                                printf(", %s", ipaddr_string(tptr));
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_MAX_BW:
-                            case LS_OPAQUE_TE_LINK_SUBTLV_MAX_RES_BW:
-                                bw.i = EXTRACT_32BITS(tptr);
-                                printf(", %.3f Mbps", bw.f*8/1000000 );
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_UNRES_BW:
-                                for (te_class = 0; te_class < 8; te_class++) {
-                                    bw.i = EXTRACT_32BITS(tptr+te_class*4);
-                                    printf("\n\t\tTE-Class %u: %.3f Mbps",
-                                           te_class,
-                                           bw.f*8/1000000 );
-                                }
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_BW_CONSTRAINTS:
-                                printf("\n\t\tBandwidth Constraints Model ID: %s (%u)",
-                                       tok2str(diffserv_te_bc_values, "unknown", *tptr),
-                                       *tptr);
-                                /* decode BCs until the subTLV ends */
-                                for (te_class = 0; te_class < (subtlv_length-4)/4; te_class++) {
-                                    bw.i = EXTRACT_32BITS(tptr+4+te_class*4);
-                                    printf("\n\t\t  Bandwidth constraint CT%u: %.3f Mbps",
-                                           te_class,
-                                           bw.f*8/1000000 );
-                                }
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_TE_METRIC:
-                                printf(", Metric %u", EXTRACT_32BITS(tptr));
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_LINK_PROTECTION_TYPE:
-                                printf(", %s, Priority %u",
-                                       bittok2str(gmpls_link_prot_values, "none", *tptr),
-                                       *(tptr+1));
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_INTF_SW_CAP_DESCR:
-                                printf("\n\t\tInterface Switching Capability: %s",
-                                       tok2str(gmpls_switch_cap_values, "Unknown", *(tptr)));
-                                printf("\n\t\tLSP Encoding: %s\n\t\tMax LSP Bandwidth:",
-                                       tok2str(gmpls_encoding_values, "Unknown", *(tptr+1)));
-                                for (priority_level = 0; priority_level < 8; priority_level++) {
-                                    bw.i = EXTRACT_32BITS(tptr+4+(priority_level*4));
-                                    printf("\n\t\t  priority level %d: %.3f Mbps",
-                                           priority_level,
-                                           bw.f*8/1000000 );
-                                }
-                                break;
-                            case LS_OPAQUE_TE_LINK_SUBTLV_LINK_TYPE:
-                                printf(", %s (%u)",
-                                       tok2str(lsa_opaque_te_tlv_link_type_sub_tlv_values,"unknown",*tptr),
-                                       *tptr);
-                                break;
-
-                            case LS_OPAQUE_TE_LINK_SUBTLV_SHARED_RISK_GROUP:
-                                count_srlg = subtlv_length / 4;
-                                if (count_srlg != 0)
-                                     printf("\n\t\t  Shared risk group: ");
-                                while (count_srlg > 0) {
-                                        bw.i = EXTRACT_32BITS(tptr);
-                                        printf("%d",bw.i);
-                                        tptr+=4;
-                                        count_srlg--;
-                                        if (count_srlg > 0)
-                                            printf(", ");
-                                }
-                                break;
-
-                            default:
-                                if (vflag <= 1) {
-                                    if(!print_unknown_data(tptr,"\n\t\t",subtlv_length))
-                                        return(ls_end);
-                                }
-                                break;
-                            }
-                            /* in OSPF everything has to be 32-bit aligned, including TLVs */
-                            if (subtlv_length%4 != 0)
-                                subtlv_length+=4-(subtlv_length%4);
-                            
-                            tlv_length-=subtlv_length;
-                            tptr+=subtlv_length;
-                            
-                        }
-                        break;
-                        
-                    case LS_OPAQUE_TE_TLV_ROUTER:
-                        if (tlv_length < 4) {
-                            printf("\n\t    TLV length %u < 4", tlv_length);
-                            return(ls_end);
-                        }
-                        TCHECK2(*tptr, 4);
-                        printf(", %s", ipaddr_string(tptr));
-                        break;
-                        
-                    default:
-                        if (vflag <= 1) {
-                            if(!print_unknown_data(tptr,"\n\t      ",tlv_length))
-                                return(ls_end);
-                        }
-                        break;
-                    }
-                    tptr+=tlv_length;
-		}
-                break;
-	    }
-	    break;
-        default:
-            if (vflag <= 1) {
-                if(!print_unknown_data((u_int8_t *)lsap->lsa_un.un_unknown,
-                                       "\n\t    ", ls_length))
+                if (ospf_print_grace_lsa((u_int8_t *)(&lsap->lsa_un.un_grace_tlv.type),
+                                         ls_length) == -1) {
                     return(ls_end);
-            } 
-            break;
+                }
+                break;
+
+	    case LS_OPAQUE_TYPE_TE:
+                if (ospf_print_te_lsa((u_int8_t *)(&lsap->lsa_un.un_te_lsa_tlv.type),
+                                      ls_length) == -1) {
+                    return(ls_end);
+                }
+                break;
+
+            default:
+                if (vflag <= 1) {
+                    if(!print_unknown_data((u_int8_t *)lsap->lsa_un.un_unknown,
+                                           "\n\t    ", ls_length))
+                        return(ls_end);
+                } 
+                break;
+            }
         }
 
         /* do we want to see an additionally hexdump ? */
@@ -745,6 +831,98 @@ trunc:
 	return (NULL);
 }
 
+static int
+ospf_decode_lls(register const struct ospfhdr *op,
+		register u_int length)
+{
+    register const u_char *dptr;
+    register const u_char *dataend;
+    register u_int length2;
+    register u_int16_t lls_type, lls_len;
+    register u_int32_t lls_flags;
+
+    switch (op->ospf_type) {
+
+    case OSPF_TYPE_HELLO:
+        if (!(op->ospf_hello.hello_options & OSPF_OPTION_L))
+            return (0);
+        break;
+
+    case OSPF_TYPE_DD:
+        if (!(op->ospf_db.db_options & OSPF_OPTION_L))
+            return (0);
+        break;
+
+    default:
+        return (0);
+    }
+
+    /* dig deeper if LLS data is available; see RFC4813 */
+    length2 = EXTRACT_16BITS(&op->ospf_len);
+    dptr = (u_char *)op + length2;
+    dataend = (u_char *)op + length;
+
+    if (EXTRACT_16BITS(&op->ospf_authtype) == OSPF_AUTH_MD5) {
+        dptr = dptr + op->ospf_authdata[3];
+        length2 += op->ospf_authdata[3];
+    }
+    if (length2 >= length) {
+        printf("\n\t[LLS truncated]");
+        return (1);
+    }
+    TCHECK2(*dptr, 2);
+    printf("\n\t  LLS: checksum: 0x%04x", (u_int)EXTRACT_16BITS(dptr));
+
+    dptr += 2;
+    TCHECK2(*dptr, 2);
+    length2 = EXTRACT_16BITS(dptr);
+    printf(", length: %u", length2);
+
+    dptr += 2;
+    TCHECK(*dptr);
+    while (dptr < dataend) {
+        TCHECK2(*dptr, 2);
+        lls_type = EXTRACT_16BITS(dptr);
+        printf("\n\t    %s (%u)",
+               tok2str(ospf_lls_tlv_values,"Unknown TLV",lls_type),
+               lls_type);
+        dptr += 2;
+        TCHECK2(*dptr, 2);
+        lls_len = EXTRACT_16BITS(dptr);
+        printf(", length: %u", lls_len);
+        dptr += 2;
+        switch (lls_type) {
+
+        case OSPF_LLS_EO:
+            if (lls_len != 4) {
+                printf(" [should be 4]");
+                lls_len = 4;
+            }
+            TCHECK2(*dptr, 4);
+            lls_flags = EXTRACT_32BITS(dptr);
+            printf("\n\t      Options: 0x%08x [%s]", lls_flags,
+                   bittok2str(ospf_lls_eo_options,"?",lls_flags));
+
+            break;
+
+        case OSPF_LLS_MD5:
+            if (lls_len != 20) {
+                printf(" [should be 20]");
+                lls_len = 20;
+            }
+			TCHECK2(*dptr, 4);
+            printf("\n\t      Sequence number: 0x%08x", EXTRACT_32BITS(dptr));
+            break;
+        }
+
+        dptr += lls_len;
+    }
+
+    return (0);
+trunc:
+    return (1);
+}
+
 static int
 ospf_decode_v2(register const struct ospfhdr *op,
     register const u_char *dataend)
@@ -765,11 +943,11 @@ ospf_decode_v2(register const struct ospfhdr *op,
 		break;
 
 	case OSPF_TYPE_HELLO:
-                printf("\n\tOptions: [%s]",
+                printf("\n\tOptions [%s]",
                        bittok2str(ospf_option_values,"none",op->ospf_hello.hello_options));
 
                 TCHECK(op->ospf_hello.hello_deadint);
-                printf("\n\t  Hello Timer: %us, Dead Timer %us, Mask: %s, Priority: %u",
+                printf("\n\t  Hello Timer %us, Dead Timer %us, Mask %s, Priority %u",
                        EXTRACT_16BITS(&op->ospf_hello.hello_helloint),
                        EXTRACT_32BITS(&op->ospf_hello.hello_deadint),
                        ipaddr_string(&op->ospf_hello.hello_mask),
@@ -797,19 +975,23 @@ ospf_decode_v2(register const struct ospfhdr *op,
 
 	case OSPF_TYPE_DD:
 		TCHECK(op->ospf_db.db_options);
-                printf("\n\tOptions: [%s]",
+                printf("\n\tOptions [%s]",
                        bittok2str(ospf_option_values,"none",op->ospf_db.db_options));
 		TCHECK(op->ospf_db.db_flags);
-                printf(", DD Flags: [%s]",
+                printf(", DD Flags [%s]",
                        bittok2str(ospf_dd_flag_values,"none",op->ospf_db.db_flags));
+                TCHECK(op->ospf_db.db_ifmtu);
+                if (op->ospf_db.db_ifmtu) {
+                        printf(", MTU: %u", ntohs(op->ospf_db.db_ifmtu));
+                }
+                TCHECK(op->ospf_db.db_seq);
+                printf(", Sequence: 0x%08x", EXTRACT_32BITS(&op->ospf_db.db_seq));
 
-		if (vflag) {
-			/* Print all the LS adv's */
-			lshp = op->ospf_db.db_lshdr;
-			while (ospf_print_lshdr(lshp) != -1) {
-				++lshp;
-			}
-		}
+                /* Print all the LS adv's */
+                lshp = op->ospf_db.db_lshdr;
+                while (((u_char *)lshp < dataend) && ospf_print_lshdr(lshp) != -1) {
+                    ++lshp;
+                }
 		break;
 
 	case OSPF_TYPE_LS_REQ:
@@ -863,7 +1045,6 @@ ospf_decode_v2(register const struct ospfhdr *op,
                 break;
 
 	default:
-		printf("v2 type (%d)", op->ospf_type);
 		break;
 	}
 	return (0);
@@ -892,25 +1073,30 @@ ospf_print(register const u_char *bp, register u_int length,
 	/* value.  If it's not valid, say so and return */
 	TCHECK(op->ospf_type);
 	cp = tok2str(type2str, "unknown LS-type", op->ospf_type);
-	printf("OSPFv%u, %s, length: %u",
+	printf("OSPFv%u, %s, length %u",
 	       op->ospf_version,
 	       cp,
 	       length);
 	if (*cp == 'u')
 		return;
 
-        if(!vflag) /* non verbose - so lets bail out here */
+        if(!vflag) { /* non verbose - so lets bail out here */
                 return;
+        }
 
 	TCHECK(op->ospf_len);
 	if (length != EXTRACT_16BITS(&op->ospf_len)) {
 		printf(" [len %d]", EXTRACT_16BITS(&op->ospf_len));
-		return;
 	}
-	dataend = bp + length;
+
+	if (length > EXTRACT_16BITS(&op->ospf_len)) {
+		dataend = bp + EXTRACT_16BITS(&op->ospf_len);
+	} else {
+		dataend = bp + length;
+	}
 
 	TCHECK(op->ospf_routerid);
-        printf("\n\tRouter-ID: %s", ipaddr_string(&op->ospf_routerid));
+        printf("\n\tRouter-ID %s", ipaddr_string(&op->ospf_routerid));
 
 	TCHECK(op->ospf_areaid);
 	if (op->ospf_areaid.s_addr != 0)
@@ -933,7 +1119,7 @@ ospf_print(register const u_char *bp, register u_int length,
 
 		case OSPF_AUTH_SIMPLE:
                         printf("\n\tSimple text password: ");
-                        safeputs(op->ospf_authdata, OSPF_AUTH_SIMPLE_LEN);
+                        safeputs((const char *)op->ospf_authdata, OSPF_AUTH_SIMPLE_LEN);
 			break;
 
 		case OSPF_AUTH_MD5:
@@ -954,6 +1140,10 @@ ospf_print(register const u_char *bp, register u_int length,
 		/* ospf version 2 */
 		if (ospf_decode_v2(op, dataend))
 			goto trunc;
+		if (length > EXTRACT_16BITS(&op->ospf_len)) {
+			if (ospf_decode_lls(op, length))
+				goto trunc;
+		}
 		break;
 
 	default:
